"""
Metrics command
Return some Authentic2 metrics as JSON
"""

import json
import logging
import sys
from datetime import timedelta

from authentic2.apps.journal.models import Event
from authentic2.backends import get_user_queryset
from authentic2.base_commands import LogToConsoleCommand
from django.contrib.sessions.models import Session
from django.db.models import Count
from django.utils import timezone

log = logging.getLogger(__name__)

DEFAULT_PERIOD_LENGTH = 300


class Command(LogToConsoleCommand):
    """
    Authentic2 metrics command
    """

    loggername = __name__

    def add_arguments(self, parser):
        """Add command argument to parser"""
        parser.add_argument("-p", "--pretty", action="store_true", help="Pretty JSON output")
        parser.add_argument(
            "-P",
            "--period-length",
            type=lambda x: timedelta(seconds=int(x)),
            help=f"Period length (in second, default: {DEFAULT_PERIOD_LENGTH})",
            default=timedelta(seconds=DEFAULT_PERIOD_LENGTH),
        )

    def core_command(self, *args, **kwargs):  # pylint: disable=unused-argument
        """Command entrypoint"""
        now = timezone.now()
        start_time = now - kwargs["period_length"]
        sessions_user_ids = [
            user_id
            for user_id in (
                session.get_decoded().get("_auth_user_id")
                for session in Session.objects.filter(expire_date__gt=now)
            )
            if user_id
        ]
        data = {
            "active_users": get_user_queryset().filter(is_active=True).count(),
            "inactive_users": get_user_queryset().filter(is_active=False).count(),
            "total_users": get_user_queryset().count(),
            "connected_users": len(set(sessions_user_ids)),
            "active_sessions": len(sessions_user_ids),
            "events": {
                row["type__name"]: row["count"]
                for row in (
                    Event.objects.filter(timestamp__gte=start_time)
                    .values("type__name")
                    .annotate(count=Count("id"))
                    .order_by("type__name")
                )
            },
        }
        data["total_events"] = sum(data["events"].values())
        json.dump(data, sys.stdout, indent=2 if kwargs["pretty"] else None)


# vim: tabstop=4 shiftwidth=4 softtabstop=4 expandtab
